#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e10|e13|e14|e15|e17|e19|c2|c5|c7> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}
sourcetar=${5}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=xrootd
origpkgver=v4_8_5
pkgver=${origpkgver}
ssibuildshims_version=v1_04_11
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname=xrootd-${pkgdotver}

get_this_dir

get_ssibuildshims

source define_basics --

if [ ${OS1} = "Darwin" ] || [ ${OS} = "u16-x86_64" ] || [ ${OS} = "u18-x86_64" ]
then
   libdir=${pkgdir}/lib
else 
   libdir=${pkgdir}/lib64
fi

if [ "${maketar}" = "tar" ] && [ -d ${libdir} ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    :
elif [ "${extraqual}" = "debug" ]
then
    relflg="-DCMAKE_BUILD_TYPE=Debug"
elif [ "${extraqual}" = "prof" ]
then
    relflg="-DCMAKE_BUILD_TYPE=RelWithDebInfo"
    cxxflg="-fno-omit-frame-pointer"
    if [ "${OS1}" = "Darwin" ]
    then
      cxxflg="${cxxflg} -fno-strict-aliasing -Wno-maybe-uninitialized"
    fi
fi

if [ "${OS1}" = "Darwin" ]
then
  cxxflg="${cxxflg} -Wno-unused-variable -Wno-unused-but-set-variable"
fi

cflg=${cxxflg}

XROOTDROOT=${pkgdir}

if [[ "${basequal}" == e1[0245] ]]; then
  cc=gcc
  cxx=g++
  cxxflg="${cxxflg} -std=c++14"
elif [[ "${basequal}" == e1[79] ]]; then
  cc=gcc
  cxx=g++
  cxxflg="${cxxflg} -std=c++17"
elif [[ "${basequal}" == e13 ]]; then
  cc=clang
  cxx=clang++
  cxxflg="${cxxflg} -std=c++14"
elif [[ "${basequal}" == c[257] ]]; then
  cc=clang
  cxx=clang++
  cxxflg="${cxxflg} -std=c++17"
else
  ssi_die "Qualifier ${basequal} not recognized"
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# setup so we get the expected compiler
setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS}

# We need cmake
source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake "${product_dir}" || \
  ssi_die "failed to setup cmake"

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
# remove existing sourcedir to avoid patch conflicts
rm -rf ${sourcedir}
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${sourcedir}
tar xf ${tardir}/${pkgtarname}.tar.gz

cd ${pkgtarname} || ssi_die "Unable to cd to ${sourcedir}/${pkgtarname}"

for patch in "${patchdir}/xrootd-${pkgdotver}.patch"; do
  patch ${DEBUG_PATCHES:+--verbose} -p1 < "${patch}" || ssi_die "application of patch ${patch} failed."
done

[[ -n "${DEBUG_PATCHES}" ]] && exit 1

cd ${blddir}

cmake_opts=(${relflg} \
  -DPYTHON_LIBRARY=${PYTHON_LIB}/${shared_prefix}${PYTHON_INCLUDE##*/}${shared_ext} \
  -DPYTHON_INCLUDE_DIR=${PYTHON_INCLUDE} \
  -DCMAKE_INSTALL_PREFIX=${XROOTDROOT} \
  -DENABLE_LIBEVENT=FALSE)

shared_prefix=lib
if [ "${OS1}" = "Darwin" ]; then
  shared_ext=.dylib
  if { ! { [[ -f /usr/include/openssl/opensslconf.h ]] || \
         [[ -f /usr/local/include/openssl/opensslconf.h ]]; }; } &&
     [[ -f /usr/local/opt/openssl/include/openssl/opensslconf.h ]]; then
    cmake_opts[${#cmake_opts[@]}]=-DOPENSSL_ROOT_DIR=/usr/local/opt/openssl
  fi
else
  shared_ext=.so
fi

CC=${cc} CXX=${cxx} CXXFLAGS="${cxxflg}" CFLAGS="${cflg}" \
 cmake ${sourcedir}/${pkgtarname} "${cmake_opts[@]}" || \
 ssi_die "cmake failed."

make -j ${ncore} || ssi_die "make failed."

make install || ssi_die "make installed."

set +x

if [ ! -d ${libdir} ] 
then
   echo "ERROR: failed to create ${libdir}"
   echo "       Something is very wrong"
   exit 1
fi

# ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "xrootd is installed at ${XROOTDROOT}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${libdir} ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
